@@ -794,6 +794,9 @@ sub input_string() {
     my $qcinput = $self->{"qcinput"};
     my $qcparse = $qcinput->{"parser"};
 
+    my $use_cints = 0;
+    my $do_cca = $qcparse->value("do_cca");
+
     printf "molecule = %s\n", $qcparse->value("molecule") if ($debug);
 
     my $symmetry = $qcinput->symmetry();
@@ -826,6 +829,25 @@ sub input_string() {
     $basis = "$basis\n  molecule = \$:molecule";
     $basis = "$basis\n)\n";
 
+    my $integrals = "";
+    if($do_cca) {
+      $integrals = "% using cca integrals";
+      $integrals = "$integrals\nintegrals<IntegralCCA>: (";
+      my $buffer_type = $qcparse->value("integral_buffer");
+      if( $buffer_type ne "opaque" && $buffer_type ne "array" ) {
+        $buffer_type = "opaque";
+      }     
+      my $int_package = $qcparse->value("integral_package");
+      if( $int_package ne "intv3" && $int_package ne "cints" ) {
+        $int_package = "intv3";
+      }
+      $integrals = "$integrals\n  integral_buffer = $buffer_type";
+      $integrals = "$integrals\n  integral_package = $int_package";
+      $integrals = "$integrals\n  evaluator_factory = MPQC.IntegralEvaluatorFactory";
+      $integrals = "$integrals\n  molecule = \$:molecule";
+      $integrals = "$integrals\n)\n";
+    }
+
     my $fixed = $qcparse->value_as_arrayref("fixed");
     my $followed = $qcparse->value_as_arrayref("followed");
     if (scalar(@{$fixed}) != 0) {
@@ -917,12 +939,18 @@ sub input_string() {
     else {
         $mole = "$mole\n  do_gradient = no";
     }
+    if($do_cca) {
+      $mole = "$mole\n  do_cca = yes";
+    }
     $mole = "$mole\n  % method for computing the molecule's energy";
     $mole = "$mole\n  mole<$method>: (";
     $mole = "$mole\n    molecule = \$:molecule";
     $mole = "$mole\n    basis = \$:basis";
     $mole = "$mole\n    coor = \$..:coor";
     $mole = "$mole\n    memory = $memory";
+    if($do_cca) {
+      $mole = "$mole\n    integrals = \$:integrals";
+    }
     if ($inputmethod eq "SCF" || $inputmethod eq "UHF"
         || $method eq "CLKS" || $method eq "UKS" || $method eq "HSOSKS") {
         $mole = "$mole\n    total_charge = $charge";
@@ -961,7 +989,8 @@ sub input_string() {
             $mole = "$mole\n    )\n";
         }
         $mole = append_reference($mole,"CLHF",$charge,$mult,$memory,$orthog_method,
-                                 $lindep_tol,$docc,$socc,1,"DZ (Dunning)");
+                                 $lindep_tol,$docc,$socc,"DZ (Dunning)");
+        $use_cints = 1;
     }
     elsif ($method eq "MBPT2") {
         my $fzc = $qcinput->fzc();
@@ -982,11 +1011,11 @@ sub input_string() {
             $refmethod = "HSOSHF";
         }
         $mole = append_reference($mole,$refmethod,$charge,$mult,$memory,$orthog_method,
-                                 $lindep_tol,$docc,$socc,0,"STO-3G");
+                                 $lindep_tol,$docc,$socc,"STO-3G");
     }
     elsif (! ($basis =~ /^STO/
               || $basis =~ /^MI/
-              || $basis =~ /^\d-\d1G$/)) {
+              || $basis =~ /^\d-\d1G$/) && ! $do_cca ) {
         my $guessmethod = "${openmethod}HF";
         $mole = "$mole\n    guess_wavefunction<$guessmethod>: (";
         $mole = "$mole\n      molecule = \$:molecule";
@@ -999,6 +1028,9 @@ sub input_string() {
         $mole = "$mole\n        name = \"STO-3G\"";
         $mole = "$mole\n      )";
         $mole = "$mole\n      memory = $memory";
+        if($do_cca) {
+          $mole = "$mole\n      integrals = \$:integrals";
+        }
         $mole = "$mole\n    )";
     }
     if ($qcinput->frequencies()) {
@@ -1063,6 +1095,9 @@ sub input_string() {
                           $mpqcstart,bool_to_yesno($qcinput->checkpoint()));
     $mpqcstart = sprintf ("%s  restart = %s\n",
                           $mpqcstart,bool_to_yesno($qcinput->restart()));
+    if ($use_cints) {
+        $mpqcstart = "$mpqcstart  integrals<IntegralCints>: ()\n";
+    }
     my $mpqcstop = ")\n";
     my $emacs = "% Emacs should use -*- KeyVal -*- mode\n";
     my $warn = "% this file was automatically generated\n";
@@ -1073,7 +1108,7 @@ sub input_string() {
         $label =~ s/\n/\n% label: /g;
         $label = "$label\n";
     }
-    "$emacs$warn$label$mol$basis$mpqcstart$coor$mole$opt$freq$mpqcstop";
+    "$emacs$warn$label$mol$basis$integrals$mpqcstart$coor$mole$opt$freq$mpqcstop";
 }
 
 sub mpqc_fixed_coor {
@@ -1154,7 +1189,6 @@ sub append_reference {
     my $lindep_tol = shift;
     my $docc = shift;
     my $socc = shift;
-    my $use_cints = shift;
     my $guessbasis = shift;
     $mole = "$mole\n    reference<$refmethod>: (";
     $mole = "$mole\n      molecule = \$:molecule";
@@ -1162,9 +1196,6 @@ sub append_reference {
     $mole = "$mole\n      total_charge = $charge";
     $mole = "$mole\n      multiplicity = $mult";
     $mole = "$mole\n      memory = $memory";
-    if ($use_cints) {
-        $mole = "$mole\n      integrals<IntegralCints>: ()";
-    }
     if ($orthog_method ne "" ) {
         $mole = "$mole\n      orthog_method = $orthog_method";
     }
@@ -1177,9 +1208,6 @@ sub append_reference {
            || $basis =~ /^MI/
            || $basis =~ /^\d-\d1G$/)) {
         $mole = "$mole\n      guess_wavefunction<$refmethod>: (";
-        if ($use_cints) {
-            $mole = "$mole\n        integrals<IntegralCints>: ()";
-        }
         $mole = "$mole\n        molecule = \$:molecule";
         $mole = "$mole\n        total_charge = $charge";
         $mole = "$mole\n        multiplicity = $mult";
