@@ -63,6 +63,7 @@
 #endif
 
 #include <util/options/GetLongOpt.h>
+#include <util/class/scexception.h>
 #include <util/misc/newstring.h>
 #include <util/keyval/keyval.h>
 #include <util/state/state_bin.h>
@@ -74,6 +75,9 @@
 #include <util/misc/bug.h>
 #include <util/misc/formio.h>
 #include <util/misc/exenv.h>
+#ifdef HAVE_CHEMISTRY_CCA
+  #include <util/misc/ccaenv.h>
+#endif
 #include <util/render/render.h>
 
 #include <math/optimize/opt.h>
@@ -86,6 +90,7 @@
 #include <chemistry/qc/wfn/wfn.h>
 
 // Force linkages:
+#include <util/group/linkage.h>
 #include <chemistry/qc/wfn/linkage.h>
 #include <chemistry/qc/scf/linkage.h>
 #include <chemistry/qc/dft/linkage.h>
@@ -104,6 +109,9 @@
 #ifdef HAVE_SC_SRC_LIB_CHEMISTRY_QC_PSI
 #  include <chemistry/qc/psi/linkage.h>
 #endif
+#ifdef HAVE_SC_SRC_LIB_CHEMISTRY_QC_INTCCA
+#  include <chemistry/qc/intcca/linkage.h>
+#endif
 
 #ifdef HAVE_MPI
 #define MPICH_SKIP_MPICXX
@@ -154,6 +162,21 @@ clean_up(void)
 #  include <fenv.h>
 #endif
 
+static void
+print_unseen(const Ref<ParsedKeyVal> &parsedkv,
+             const char *input)
+{
+  if (parsedkv->have_unseen()) {
+    ExEnv::out0() << endl;
+    ExEnv::out0() << indent
+         << "The following keywords in \"" << input << "\" were ignored:"
+         << endl;
+    ExEnv::out0() << incindent;
+    parsedkv->print_unseen(ExEnv::out0());
+    ExEnv::out0() << decindent;
+  }
+}
+
 int
 try_main(int argc, char *argv[])
 {
@@ -228,6 +251,10 @@ try_main(int argc, char *argv[])
   options.enroll("i", GetLongOpt::NoValue, "convert simple to OO input", 0);
   options.enroll("d", GetLongOpt::NoValue, "debug", 0);
   options.enroll("h", GetLongOpt::NoValue, "print this message", 0);
+  options.enroll("cca-path", GetLongOpt::OptionalValue, 
+                 "cca component path", "");
+  options.enroll("cca-load", GetLongOpt::OptionalValue,
+                 "cca components to load", "");
 
   int optind = options.parse(argc, argv);
 
@@ -370,9 +397,12 @@ try_main(int argc, char *argv[])
   Ref<KeyVal> keyval = new PrefixKeyVal(parsedkv.pointer(),"mpqc");
 
   // get the basename for output files
-  int nfilebase = (int) (::strrchr(input, '.') - input);
+  const char *basename_source;
+  if (output) basename_source = output;
+  else        basename_source = input;
+  int nfilebase = (int) (::strrchr(basename_source, '.') - basename_source);
   char *basename = new char[nfilebase + 1];
-  strncpy(basename, input, nfilebase);
+  strncpy(basename, basename_source, nfilebase);
   basename[nfilebase] = '\0';
   SCFormIO::set_default_basename(basename);
 
@@ -467,6 +497,38 @@ try_main(int argc, char *argv[])
        << indent
        << "Total number of processors = " << grp->n() * thread->nthread() << endl;
 
+#ifdef HAVE_CHEMISTRY_CCA
+  // initialize cca framework
+  KeyValValuestring emptystring("");
+  bool do_cca = keyval->booleanvalue("do_cca",falsevalue);
+
+  string cca_path(options.retrieve("cca-path"));
+  string cca_load(options.retrieve("cca-load"));
+  if(cca_path.size()==0)
+    cca_path = keyval->stringvalue("cca_path",emptystring);
+  if(cca_load.size()==0)
+    cca_load = keyval->stringvalue("cca_load",emptystring);
+
+  if( !do_cca && (cca_load.size() > 0 || cca_path.size() > 0) )
+    do_cca = true;
+
+  if(cca_path.size()==0) {
+    #ifdef CCA_PATH
+      cca_path = CCA_PATH;                                 
+    #endif
+  }
+  if(cca_load.size()==0) {
+    cca_load += "MPQC.IntegralEvaluatorFactory";
+  }
+
+  if( cca_load.size() > 0 && cca_path.size() > 0 && do_cca ) {
+    string cca_args = "--path " + cca_path + " --load " + cca_load;
+    ExEnv::out0() << endl << indent << "Initializing CCA framework with args: "
+                  << endl << indent << cca_args << endl;
+    CCAEnv::init( cca_args );
+  }
+#endif
+
   // now set up the debugger
   Ref<Debugger> debugger; debugger << keyval->describedclassvalue("debug");
   if (debugger.nonnull()) {
@@ -485,10 +547,18 @@ try_main(int argc, char *argv[])
 
   // get the integral factory. first try commandline and environment
   Ref<Integral> integral = Integral::initial_integral(argc, argv);
+  
+  // if we still don't have a integral, try reading the integral
+  // from the input
+  if (integral.null()) {
+    integral << keyval->describedclassvalue("integrals");
+  }
+
   if (integral.nonnull())
     Integral::set_default_integral(integral);
   else
     integral = Integral::get_default_integral();
+
   ExEnv::out0() << endl << indent
        << "Using " << integral->class_name()
        << " by default for molecular integrals evaluation" << endl << endl;
@@ -573,6 +643,7 @@ try_main(int argc, char *argv[])
       mole->set_checkpoint_freq(checkpoint_freq);
     }
   }
+  delete[] mole_ckpt_file;
 
   if (checkpoint && opt.nonnull()) {
     opt->set_checkpoint();
@@ -618,7 +689,23 @@ try_main(int argc, char *argv[])
   int print_mole = keyval->booleanvalue("print_mole",truevalue);
   
   int print_timings = keyval->booleanvalue("print_timings",truevalue);
-  
+
+  // see if any pictures are desired
+  Ref<Render> renderer;
+  renderer << keyval->describedclassvalue("renderer");
+
+  // If we have a renderer, then we will read in some more info
+  // below.  Otherwise we can get rid of the keyval's, to eliminate
+  // superfluous references to objects that we might otherwise be
+  // able to delete.  We cannot read in the remaining rendering
+  // objects now, since some of their KeyVal CTOR's are heavyweight,
+  // requiring optimized geometries, etc.
+  if (renderer.null()) {
+    if (parsedkv.nonnull()) print_unseen(parsedkv, input);
+    keyval = 0;
+    parsedkv = 0;
+  }
+
   // sanity checks for the benefit of reasonable looking output
   if (opt.null()) do_opt=0;
   
@@ -641,7 +728,7 @@ try_main(int argc, char *argv[])
 
   delete[] restartfile;
   delete[] ckptfile;
-  
+
   int ready_for_freq = 1;
   if (mole.nonnull()) {
     if (((do_opt && opt.nonnull()) || do_grad)
@@ -746,9 +833,9 @@ try_main(int argc, char *argv[])
       SavableState::save_state(mole.pointer(),so);
       so.close();
 
-      delete[] wfn_file;
     }
   }
+  delete[] wfn_file;
 
   // Frequency calculation.
   if (ready_for_freq && molfreq.nonnull()) {
@@ -783,57 +870,56 @@ try_main(int argc, char *argv[])
     }
   }
 
-  // see if any pictures are desired
-  Ref<Render> renderer;
-  renderer << keyval->describedclassvalue("renderer");
-  Ref<RenderedObject> rendered;
-  rendered << keyval->describedclassvalue("rendered");
-  Ref<AnimatedObject> animated;
-  animated << keyval->describedclassvalue("rendered");
-  if (renderer.nonnull() && rendered.nonnull()) {
-    if (tim.nonnull()) tim->enter("render");
-    if (grp->me() == 0) renderer->render(rendered);
-    if (tim.nonnull()) tim->exit("render");
-  }
-  else if (renderer.nonnull() && animated.nonnull()) {
-    if (tim.nonnull()) tim->enter("render");
-    if (grp->me() == 0) renderer->animate(animated);
-    if (tim.nonnull()) tim->exit("render");
-  }
-  else if (renderer.nonnull()) {
-    if (tim.nonnull()) tim->enter("render");
-    int n = keyval->count("rendered");
-    for (i=0; i<n; i++) {
-      rendered << keyval->describedclassvalue("rendered",i);
-      animated << keyval->describedclassvalue("rendered",i);
-      if (rendered.nonnull()) {
-        // make sure the object has a name so we don't overwrite its file
-        if (rendered->name() == 0) {
-          char ic[64];
-          sprintf(ic,"%02d",i);
-          rendered->set_name(ic);
+  if (renderer.nonnull()) {
+    Ref<RenderedObject> rendered;
+    rendered << keyval->describedclassvalue("rendered");
+    Ref<AnimatedObject> animated;
+    animated << keyval->describedclassvalue("rendered");
+    if (rendered.nonnull()) {
+      if (tim.nonnull()) tim->enter("render");
+      if (grp->me() == 0) renderer->render(rendered);
+      if (tim.nonnull()) tim->exit("render");
+    }
+    else if (animated.nonnull()) {
+      if (tim.nonnull()) tim->enter("render");
+      if (grp->me() == 0) renderer->animate(animated);
+      if (tim.nonnull()) tim->exit("render");
+    }
+    else {
+      if (tim.nonnull()) tim->enter("render");
+      int n = keyval->count("rendered");
+      for (i=0; i<n; i++) {
+        rendered << keyval->describedclassvalue("rendered",i);
+        animated << keyval->describedclassvalue("rendered",i);
+        if (rendered.nonnull()) {
+          // make sure the object has a name so we don't overwrite its file
+          if (rendered->name() == 0) {
+            char ic[64];
+            sprintf(ic,"%02d",i);
+            rendered->set_name(ic);
+          }
+          if (grp->me() == 0) renderer->render(rendered);
         }
-        if (grp->me() == 0) renderer->render(rendered);
-      }
-      else if (animated.nonnull()) {
-        // make sure the object has a name so we don't overwrite its file
-        if (animated->name() == 0) {
-          char ic[64];
-          sprintf(ic,"%02d",i);
-          animated->set_name(ic);
+        else if (animated.nonnull()) {
+          // make sure the object has a name so we don't overwrite its file
+          if (animated->name() == 0) {
+            char ic[64];
+            sprintf(ic,"%02d",i);
+            animated->set_name(ic);
+          }
+          if (grp->me() == 0) renderer->animate(animated);
         }
-        if (grp->me() == 0) renderer->animate(animated);
       }
+      if (tim.nonnull()) tim->exit("render");
+    }
+    Ref<MolFreqAnimate> molfreqanim;
+    molfreqanim << keyval->describedclassvalue("animate_modes");
+    if (ready_for_freq && molfreq.nonnull()
+        && molfreqanim.nonnull()) {
+      if (tim.nonnull()) tim->enter("render");
+      molfreq->animate(renderer, molfreqanim);
+      if (tim.nonnull()) tim->exit("render");
     }
-    if (tim.nonnull()) tim->exit("render");
-  }
-  Ref<MolFreqAnimate> molfreqanim;
-  molfreqanim << keyval->describedclassvalue("animate_modes");
-  if (ready_for_freq && molfreq.nonnull()
-      && molfreqanim.nonnull() && renderer.nonnull()) {
-    if (tim.nonnull()) tim->enter("render");
-    molfreq->animate(renderer, molfreqanim);
-    if (tim.nonnull()) tim->exit("render");
   }
 
   if (mole.nonnull()) {
@@ -855,15 +941,7 @@ try_main(int argc, char *argv[])
          << endl;
   }
 
-  if (parsedkv->have_unseen()) {
-    ExEnv::out0() << indent
-         << "The following keywords in \"" << input << "\" were ignored:"
-         << endl;
-    ExEnv::out0() << incindent;
-    parsedkv->print_unseen(ExEnv::out0());
-    ExEnv::out0() << decindent;
-    ExEnv::out0() << endl;
-  }
+  if (parsedkv.nonnull()) print_unseen(parsedkv, input);
 
   if (print_timings)
     if (tim.nonnull()) tim->print(ExEnv::out0());
@@ -872,9 +950,6 @@ try_main(int argc, char *argv[])
   delete[] molname;
   SCFormIO::set_default_basename(0);
 
-  molfreqanim = 0;
-  animated = 0;
-  rendered = 0;
   renderer = 0;
   molfreq = 0;
   molhess = 0;
@@ -914,6 +989,13 @@ main(int argc, char *argv[])
   try {
     try_main(argc, argv);
   }
+  catch (SCException &e) {
+    cout << argv[0] << ": ERROR: SC EXCEPTION RAISED:" << endl
+         << e.what()
+         << endl;
+    clean_up();
+    throw;
+  }
   catch (bad_alloc &e) {
     cout << argv[0] << ": ERROR: MEMORY ALLOCATION FAILED:" << endl
          << e.what()
