@@ -33,6 +33,7 @@
 #include <string.h>
 #include <stdio.h>
 
+#include <util/class/scexception.h>
 #include <util/misc/formio.h>
 #include <util/state/stateio.h>
 #include <chemistry/molecule/molecule.h>
@@ -47,7 +48,7 @@ using namespace sc;
 // Molecule
 
 static ClassDesc Molecule_cd(
-  typeid(Molecule),"Molecule",5,"public SavableState",
+  typeid(Molecule),"Molecule",6,"public SavableState",
   create<Molecule>, create<Molecule>, create<Molecule>);
 
 Molecule::Molecule():
@@ -60,6 +61,9 @@ Molecule::Molecule():
   equiv_ = 0;
   nequiv_ = 0;
   atom_to_uniq_ = 0;
+  q_Z_ = atominfo_->string_to_Z("Q");
+  include_q_ = false;
+  include_qq_ = false;
   init_symmetry_info();
 }
 
@@ -111,8 +115,13 @@ Molecule::Molecule(const Ref<KeyVal>&input):
   nequiv_ = 0;
   atom_to_uniq_ = 0;
 
+  KeyValValueboolean kvfalse(0);
+  include_q_ = input->booleanvalue("include_q",kvfalse);
+  include_qq_ = input->booleanvalue("include_qq",kvfalse);
+
   atominfo_ << input->describedclassvalue("atominfo");
   if (atominfo_.null()) atominfo_ = new AtomInfo;
+  q_Z_ = atominfo_->string_to_Z("Q");
   if (input->exists("pdb_file")) {
       geometry_units_ = new Units("angstrom");
       char* filename = input->pcharvalue("pdb_file");
@@ -143,9 +152,8 @@ Molecule::Molecule(const Ref<KeyVal>&input):
       // possible
       int natom = input->count("geometry");
       if (natom != input->count("atoms")) {
-          ExEnv::err0() << indent
-               << "Molecule: size of \"geometry\" != size of \"atoms\"\n";
-          abort();
+          throw InputError("size of \"geometry\" != size of \"atoms\"",
+                           __FILE__, __LINE__, 0, 0, class_desc());
         }
 
       int i;
@@ -158,7 +166,7 @@ Molecule::Molecule(const Ref<KeyVal>&input):
               have_charge = 1;
               charge = 0.0;
             }
-          add_atom(AtomInfo::string_to_Z(name = input->pcharvalue("atoms",i)),
+          add_atom(atominfo_->string_to_Z(name = input->pcharvalue("atoms",i)),
                    input->doublevalue("geometry",i,0)*conv,
                    input->doublevalue("geometry",i,1)*conv,
                    input->doublevalue("geometry",i,2)*conv,
@@ -215,6 +223,12 @@ Molecule::operator=(const Molecule& mol)
   atominfo_ = mol.atominfo_;
   geometry_units_ = new Units(mol.geometry_units_->string_rep());
 
+  q_Z_ = mol.q_Z_;
+  include_q_ = mol.include_q_;
+  include_qq_ = mol.include_qq_;
+  q_atoms_ = mol.q_atoms_;
+  non_q_atoms_ = mol.non_q_atoms_;
+
   natoms_ = mol.natoms_;
 
   if (natoms_) {
@@ -340,6 +354,13 @@ Molecule::add_atom(int Z,double x,double y,double z,
       charges_[natoms_] = Z;
     }
 
+  if (Z == q_Z_) {
+      q_atoms_.push_back(natoms_);
+    }
+  else {
+      non_q_atoms_.push_back(natoms_);
+    }
+
   natoms_++;
 }
 
@@ -380,7 +401,8 @@ Molecule::print_parsedkeyval(ostream& os,
   for (i=0; i<natom(); i++) {
       os << indent;
       if (number_atoms) os << scprintf(" %3d", i+1);
-      os << scprintf(" %5s", AtomInfo::symbol(Z_[i]));
+      std::string symbol(atom_symbol(i));
+      os << scprintf(" %5s", symbol.c_str());
       if (labels_) {
           const char *lab = labels_[i];
           if (lab == 0) lab = "";
@@ -462,16 +484,24 @@ Molecule::charge(int iatom) const
 double
 Molecule::nuclear_charge() const
 {
-  int i;
   double c = 0.0;
-  for (i=0; i<natom(); i++) {
+  if (include_q_) {
+    for (int i=0; i<natom(); i++) {
       c += charge(i);
     }
+  }
+  else {
+    for (int ii=0; ii<non_q_atoms_.size(); ii++) {
+      c += charge(non_q_atoms_[ii]);
+    }
+  }
   return c;
 }
 
 void Molecule::save_data_state(StateOut& so)
 {
+  so.put(include_q_);
+  so.put(include_qq_);
   so.put(natoms_);
   SavableState::save_state(pg_.pointer(),so);
   SavableState::save_state(geometry_units_.pointer(),so);
@@ -504,9 +534,19 @@ Molecule::Molecule(StateIn& si):
   natoms_(0), r_(0), Z_(0), mass_(0), labels_(0)
 {
   if (si.version(::class_desc<Molecule>()) < 4) {
-      ExEnv::errn() << "Molecule: cannot restore from old molecules" << endl;
-      abort();
+      throw FileOperationFailed("cannot restore from old molecules",
+                                __FILE__, __LINE__, 0,
+                                FileOperationFailed::Corrupt,
+                                class_desc());
     }
+  if (si.version(::class_desc<Molecule>()) < 6) {
+    include_q_ = false;
+    include_qq_ = false;
+    }
+  else {
+    si.get(include_q_);
+    si.get(include_qq_);
+  }
   si.get(natoms_);
   pg_ << SavableState::restore_state(si);
   geometry_units_ << SavableState::restore_state(si);
@@ -540,6 +580,15 @@ Molecule::Molecule(StateIn& si):
         }
     }
 
+  for (int i=0; i<natoms_; i++) {
+      if (Z_[i] == q_Z_) {
+          q_atoms_.push_back(i);
+        }
+      else {
+          non_q_atoms_.push_back(i);
+        }
+    }
+
   nuniq_ = 0;
   equiv_ = 0;
   nequiv_ = 0;
@@ -610,17 +659,47 @@ Molecule::center_of_mass() const
 double
 Molecule::nuclear_repulsion_energy()
 {
-  int i, j;
   double e=0.0;
 
-  for (i=1; i < natoms_; i++) {
-    SCVector3 ai(r(i));
-    double Zi = charge(i);
+  // non_q non_q terms
+  for (int ii=1; ii < non_q_atoms_.size(); ii++) {
+      int i = non_q_atoms_[ii];
+      SCVector3 ai(r(i));
+      double Zi = charge(i);
     
-    for (j=0; j < i; j++) {
-        SCVector3 aj(r(j));
-        e += Zi * charge(j) / ai.dist(aj);
-      }
+      for (int jj=0; jj < ii; jj++) {
+          int j = non_q_atoms_[jj];
+          SCVector3 aj(r(j));
+          e += Zi * charge(j) / ai.dist(aj);
+        }
+    }
+
+  // non_q q terms
+  for (int ii=0; ii < q_atoms_.size(); ii++) {
+      int i = q_atoms_[ii];
+      SCVector3 ai(r(i));
+      double Zi = charge(i);
+    
+      for (int jj=0; jj < non_q_atoms_.size(); jj++) {
+          int j = non_q_atoms_[jj];
+          SCVector3 aj(r(j));
+          e += Zi * charge(j) / ai.dist(aj);
+        }
+    }
+
+  if (include_qq_) {
+      // q q terms
+      for (int ii=1; ii < q_atoms_.size(); ii++) {
+          int i = q_atoms_[ii];
+          SCVector3 ai(r(i));
+          double Zi = charge(i);
+    
+          for (int jj=0; jj < ii; jj++) {
+              int j = q_atoms_[jj];
+              SCVector3 aj(r(j));
+              e += Zi * charge(j) / ai.dist(aj);
+            }
+        }
     }
 
   return e;
@@ -636,75 +715,123 @@ Molecule::nuclear_repulsion_1der(int center, double xyz[3])
   xyz[0] = 0.0;
   xyz[1] = 0.0;
   xyz[2] = 0.0;
-  for (i=0; i < natoms_; i++) {
-      SCVector3 ai(r(i));
-      double Zi = charge(i);
 
-      for (j=0; j < i; j++) {
-          if (center==i || center==j) {
-              SCVector3 aj(r(j));
+  SCVector3 r_center(r(center));
+  double Z_center = charge(center);
+  bool center_is_Q = (atom_symbol(center) == "Q");
 
-              r2 = 0.0;
-              for (k=0; k < 3; k++) {
-                  rd[k] = ai[k] - aj[k];
-                  r2 += rd[k]*rd[k];
-                }
-        
-              factor = - Zi * charge(j) * pow(r2,-1.5);
-              if (center==j) factor = -factor;
-              for (k=0; k<3; k++) {
-                  xyz[k] += factor * rd[k];
-                }
-            }
-        }
+  // this handles center = Q or non_Q and atom = non_Q
+  for (int ii=0; ii < non_q_atoms_.size(); ii++) {
+    int i = non_q_atoms_[ii];
+    if (i == center) continue;
+    SCVector3 r_i(r(i));
+
+    r2 = 0.0;
+    for (k=0; k < 3; k++) {
+      rd[k] = r_center[k] - r_i[k];
+      r2 += rd[k]*rd[k];
     }
+    factor = - Z_center * charge(i) * pow(r2,-1.5);
+    for (k=0; k<3; k++) {
+      xyz[k] += factor * rd[k];
+    }
+  }
+
+  // this handles center = Q or non_Q and atom = Q
+  for (int ii=0; ii < q_atoms_.size(); ii++) {
+    int i = q_atoms_[ii];
+    if (i == center || (!include_qq_ && center_is_Q)) continue;
+    SCVector3 r_i(r(i));
+
+    r2 = 0.0;
+    for (k=0; k < 3; k++) {
+      rd[k] = r_center[k] - r_i[k];
+      r2 += rd[k]*rd[k];
+    }
+    factor = - Z_center * charge(i) * pow(r2,-1.5);
+    for (k=0; k<3; k++) {
+      xyz[k] += factor * rd[k];
+    }
+  }
 }
 
 void
 Molecule::nuclear_charge_efield(const double *charges,
                                 const double *position, double *efield)
 {
-  int i,j;
   double tmp;
   double rd[3];
 
-  for (i=0; i<3; i++) efield[i] = 0.0;
+  for (int i=0; i<3; i++) efield[i] = 0.0;
 
-  for (i=0; i<natoms_; i++) {
+  if (include_q_) {
+    for (int i=0; i<natoms_; i++) {
       SCVector3 a(r(i));
       tmp = 0.0;
-      for (j=0; j<3; j++) {
-          rd[j] = position[j] - a[j];
-          tmp += rd[j]*rd[j];
-        }
+      for (int j=0; j<3; j++) {
+        rd[j] = position[j] - a[j];
+        tmp += rd[j]*rd[j];
+      }
       tmp = charges[i]/(tmp*sqrt(tmp));
-      for (j=0; j<3; j++) {
-          efield[j] +=  rd[j] * tmp;
-        }
+      for (int j=0; j<3; j++) {
+        efield[j] +=  rd[j] * tmp;
+      }
     }
+  }
+  else {
+    for (int ii=0; ii<non_q_atoms_.size(); ii++) {
+      int i = non_q_atoms_[ii];
+      SCVector3 a(r(i));
+      tmp = 0.0;
+      for (int j=0; j<3; j++) {
+        rd[j] = position[j] - a[j];
+        tmp += rd[j]*rd[j];
+      }
+      tmp = charges[i]/(tmp*sqrt(tmp));
+      for (int j=0; j<3; j++) {
+        efield[j] +=  rd[j] * tmp;
+      }
+    }
+  }
 }
 
 void
 Molecule::nuclear_efield(const double *position, double *efield)
 {
-  int i,j;
   double tmp;
   double rd[3];
 
-  for (i=0; i<3; i++) efield[i] = 0.0;
+  for (int i=0; i<3; i++) efield[i] = 0.0;
 
-  for (i=0; i<natoms_; i++) {
+  if (include_q_) {
+    for (int i=0; i<natoms_; i++) {
       SCVector3 a(r(i));
       tmp = 0.0;
-      for (j=0; j<3; j++) {
-          rd[j] = position[j] - a[j];
-          tmp += rd[j]*rd[j];
-        }
+      for (int j=0; j<3; j++) {
+        rd[j] = position[j] - a[j];
+        tmp += rd[j]*rd[j];
+      }
       tmp = charge(i)/(tmp*sqrt(tmp));
-      for (j=0; j<3; j++) {
-          efield[j] +=  rd[j] * tmp;
-        }
+      for (int j=0; j<3; j++) {
+        efield[j] +=  rd[j] * tmp;
+      }
     }
+  }
+  else {
+    for (int ii=0; ii<non_q_atoms_.size(); ii++) {
+      int i = non_q_atoms_[ii];
+      SCVector3 a(r(i));
+      tmp = 0.0;
+      for (int j=0; j<3; j++) {
+        rd[j] = position[j] - a[j];
+        tmp += rd[j]*rd[j];
+      }
+      tmp = charge(i)/(tmp*sqrt(tmp));
+      for (int j=0; j<3; j++) {
+        efield[j] +=  rd[j] * tmp;
+      }
+    }
+  }
 }
 
 int
@@ -774,8 +901,10 @@ Molecule::symmetrize(double tol)
       else {
         if (Z(i) != newmol->Z(atom)
             || fabs(mass(i)-newmol->mass(atom))>1.0e-10) {
-            ExEnv::err0() << "Molecule: symmetrize: atom mismatch" << endl;
-            abort();
+            throw ToleranceExceeded("symmetrize: atom mismatch",
+                                    __FILE__, __LINE__,
+                                    1.0e-10, fabs(mass(i)-newmol->mass(atom)),
+                                    class_desc());
         }
       }
     }
@@ -990,12 +1119,17 @@ Molecule::cleanup_molecule(double tol)
                 }
             }
           if (!found) {
-              ExEnv::err0()
-                   << "Molecule: cleanup: couldn't find atom at " << np << endl
-                   << "  transforming uniq atom " << i << " at " << up << endl
-                   << "  with symmetry op " << g << ":" << endl;
-              so.print(ExEnv::err0());
-              abort();
+              SCException ex("cleanup: couldn't find atom",
+                             __FILE__, __LINE__, class_desc());
+              try {
+                  ex.elaborate()
+                      << "couldn't find atom at " << np << endl
+                      << "transforming uniq atom " << i << " at " << up << endl
+                      << "with symmetry op " << g << ":" << endl;
+                  so.print(ex.elaborate());
+                }
+              catch (...) {}
+              throw ex;
             }
         }
     }
@@ -1080,9 +1214,8 @@ Molecule::n_core_electrons()
       if (z > 48) n += 10;
       if (z > 54) n += 8;
       if (z > 72) {
-          ExEnv::errn() << "Molecule::n_core_electrons: atomic number too large"
-               << endl;
-          abort();
+          throw LimitExceeded<int>("n_core_electrons: atomic number too large",
+                                   __FILE__, __LINE__, 72, z, class_desc());
         }
     }
   return n;
@@ -1177,7 +1310,7 @@ Molecule::read_pdb(const char *filename)
               element[1] = '\0';
             }
 
-          int Z = AtomInfo::string_to_Z(element);
+          int Z = atominfo_->string_to_Z(element);
 
           char field[9];
           strncpy(field,&line[30],8); field[8] = '\0';
@@ -1214,7 +1347,8 @@ Molecule::print_pdb(ostream& os, char *title) const
   int i;
   for (i=0; i < natom(); i++) {
     char symb[4];
-    sprintf(symb,"%s1",AtomInfo::symbol(Z_[i]));
+    std::string symbol(atom_symbol(i));
+    sprintf(symb,"%s1",symbol.c_str());
 
     os << scprintf(
         "HETATM%5d  %-3s UNK %5d    %8.3f%8.3f%8.3f  0.00  0.00   0\n",
@@ -1261,6 +1395,18 @@ Molecule::label(int atom) const
   return labels_[atom];
 }
 
+std::string
+Molecule::atom_name(int iatom) const
+{
+  return atominfo_->name(Z_[iatom]);
+}
+
+std::string
+Molecule::atom_symbol(int iatom) const
+{
+  return atominfo_->symbol(Z_[iatom]);
+}
+
 /////////////////////////////////////////////////////////////////////////////
 
 // Local Variables:
