@@ -71,11 +71,39 @@ molecule<Molecule>: (
     )
   )
 </pre>
-The default units are Bohr with can be overridden with
-#unit=angstrom#.  The #atom_labels# array can be
-omitted.  The #atoms# and #geometry# arrays
+The default units are Bohr which can be overridden with
+<tt>unit=angstrom</tt>.  The <tt>atom_labels</tt> array can be
+omitted.  The <tt>atoms</tt> and <tt>geometry</tt> arrays
 are required.
 
+As a special case, an atom can be given with the symbol <tt>Q</tt> or the
+name <tt>charge</tt>.  Such centers are treated as point charges and not
+given basis functions.  The values of the charges must be specified with a
+<tt>charge</tt> vector in the Molecule input.  Since the charge vector
+assign charges to all centers, including atoms, it is easiest to place all
+point charge centers first in the geometry, and then give a charge vector
+with a number of elements equal to the number of point charges.  The
+following example shows a water molecule interacting with a point charge
+having value 0.1:
+<pre>
+molecule<Molecule>: (
+    unit=angstrom
+    charge = [ 0.1 ]
+    { atom_labels atoms           geometry            } = {
+          Q1         Q   [ 0.0         0 10.0         ]
+          O1         O   [ 0.000000000 0  0.369372944 ]
+          H1         H   [ 0.783975899 0 -0.184686472 ]
+          H2         H   [-0.783975899 0 -0.184686472 ]
+     }
+    )
+  )
+</pre>
+
+This feature is designed for doing QM/MM calculations, so, by default,
+methods will not include interactions between the <tt>Q</tt> centers when
+computing the energy or the gradient.  To include these interactions, set
+<tt>include_qq=1</tt>.
+
 The Molecule class has a PointGroup
 member object, which also has a KeyVal constructor
 that is called when a Molecule is made.  The
@@ -119,6 +147,21 @@ class Molecule: public SavableState
     double *mass_;
     char **labels_;
 
+    // The Z that represents a "Q" type atom.
+    int q_Z_;
+
+    // If true, include the q terms in the charge and efield routines
+    bool include_q_;
+
+    // If true, include the coupling between q-q pairs when
+    // computing nuclear repulsion energy and gradients.
+    bool include_qq_;
+
+    // These vectors contain the atom indices of atoms that are not type
+    // "Q" and those that are.
+    std::vector<int> q_atoms_;
+    std::vector<int> non_q_atoms_;
+
     void clear();
   public:
     Molecule();
@@ -272,9 +315,37 @@ class Molecule: public SavableState
     /// Return the maximum atomic number.
     int max_z();
 
-    /// Return the molecules AtomInfo object.
+    /// Return the molecule's AtomInfo object.
     Ref<AtomInfo> atominfo() const { return atominfo_; }
 
+    /// Returns the element name of the atom.
+    std::string atom_name(int iatom) const;
+
+    /// Returns the element symbol of the atom.
+    std::string atom_symbol(int iatom) const;
+
+    /** If include_q is true, then include the "Q" atoms in the charge and
+        efield routines. */
+    void set_include_q(bool iq) { include_q_ = iq; }
+    /// Returns include_q.  See set_include_q.
+    bool include_q() const { return include_q_; }
+
+    /** If include_qq is true, include the coupling between pairs of "Q"
+        atoms when computing nuclear repulsion energy and gradients. */
+    void set_include_qq(bool iqq) { include_qq_ = iqq; }
+    /// Returns include_qq.  See set_include_qq.
+    bool include_qq() const { return include_qq_; }
+
+    /// Retrieve the number of "Q" atoms.
+    int n_q_atom() const { return q_atoms_.size(); }
+    /// Retrieve the "Q" atoms.
+    int q_atom(int i) const { return q_atoms_[i]; }
+
+    /// Retrieve the number of non-"Q" atoms.
+    int n_non_q_atom() const { return non_q_atoms_.size(); }
+    /// Retrieve the of non-"Q" atoms.
+    int non_q_atom(int i) const { return non_q_atoms_[i]; }
+
     void save_data_state(StateOut&);
 };
 
