@@ -37,6 +37,10 @@ namespace sc {
     enum StandardApproximation {StdApprox_A = 0,
 				StdApprox_Ap = 1,
 				StdApprox_B = 2};
+    enum ABSMethod {ABS_ABS = 0,
+		    ABS_ABSPlus = 1,
+		    ABS_CABS = 2,
+		    ABS_CABSPlus = 3};
   }
 
 }
