@@ -83,7 +83,7 @@ class SCF: public OneBodyWavefunction {
     virtual void compute();
 
     // calculate the scf vector, returning the accuracy
-    virtual double compute_vector(double&);
+    virtual double compute_vector(double&, double enuclear);
 
     // return the DIIS error matrices
     virtual Ref<SCExtrapError> extrap_error();
@@ -250,6 +250,11 @@ class SCF: public OneBodyWavefunction {
     // do setup for hessian calculation
     virtual void init_hessian() =0;
     virtual void done_hessian() =0;
+
+  private:
+    // This experimental function does SVD of Coulomb matrix
+    // to be used in low-rank reconstruction
+    void svd_product_basis();
 };
 
 }
