@@ -106,7 +106,7 @@ SCF::savestate_iter(int iter)
 
 }
 double
-SCF::compute_vector(double& eelec)
+SCF::compute_vector(double& eelec, double nucrep)
 {
   tim_enter("vector");
   int i;
@@ -126,12 +126,6 @@ SCF::compute_vector(double& eelec)
 
   // set up subclass for vector calculation
   init_vector();
-  
-  // calculate the nuclear repulsion energy
-  double nucrep = molecule()->nuclear_repulsion_energy();
-  ExEnv::out0() << indent
-       << scprintf("nuclear repulsion energy = %15.10f", nucrep)
-       << endl << endl;
 
   RefDiagSCMatrix evals(oso_dimension(), basis_matrixkit());
 
@@ -139,6 +133,9 @@ SCF::compute_vector(double& eelec)
   int iter, iter_since_reset = 0;
   double accuracy = 1.0;
 
+  ExEnv::out0() << indent
+                << "Beginning iterations.  Basis is "
+                << basis()->label() << '.' << std::endl;
   for (iter=0; iter < maxiter_; iter++, iter_since_reset++) {
     // form the density from the current vector 
     tim_enter("density");
@@ -316,7 +313,7 @@ SCF::compute_vector(double& eelec)
 
     savestate_iter(iter);
   }
-    
+ 
   eigenvalues_ = evals;
   eigenvalues_.computed() = 1;
   eigenvalues_.set_actual_accuracy(accuracy<delta?delta:accuracy);
@@ -393,6 +390,9 @@ SCF::compute_vector(double& eelec)
   oso_eigenvectors_ = oso_scf_vector_;
   oso_eigenvectors_.computed() = 1;
   oso_eigenvectors_.set_actual_accuracy(delta);
+  // Checkpoint wavefunction, if needed, so that if converged
+  // on the last iteration then the wavefunction is marked as computed
+  savestate_iter(iter-1);
 
   // now clean up
   done_vector();
