@@ -28,8 +28,6 @@
 #ifndef _chemistry_qc_wfn_linkage_h
 #define _chemistry_qc_wfn_linkage_h
 
-#ifndef __PIC__
-
 #include <chemistry/qc/wfn/density.h>
 #include <chemistry/qc/wfn/orbital.h>
 #include <chemistry/qc/wfn/solvent.h>
@@ -44,6 +42,4 @@ static ForceLink<ExtendedHuckelWfn> wfn_force_link_d_;
 
 }
 
-#endif /* __PIC__ */
-
 #endif
