@@ -28,8 +28,6 @@
 #ifndef _math_optimize_linkage_h
 #define _math_optimize_linkage_h
 
-#ifndef __PIC__
-
 #include <math/optimize/qnewton.h>
 #include <math/optimize/newton.h>
 #include <math/optimize/gdiis.h>
@@ -51,6 +49,4 @@ static ForceLink<NewtonOpt> optimize_force_link_g_;
 
 }
 
-#endif /* __PIC__ */
-
 #endif
