@@ -114,24 +114,62 @@ class Optimize: virtual public SavableState {
     Ref<Function> function() const { return function_; }
     Ref<SCMatrixKit> matrixkit() const { return function_->matrixkit(); }
     RefSCDimension dimension() const { return function_->dimension(); }
+
 };
 
 
 /** The LineOpt abstract class is used to perform one dimensional
 optimizations.*/
 class LineOpt: public Optimize {
+
   protected:
+
+    double decrease_factor_;
+    RefSCVector initial_x_;
+    double initial_value_;
+    RefSCVector initial_grad_;
     RefSCVector search_direction_;
+    Ref<Function> function_;
+    
+    int sufficient_decrease(RefSCVector& step);
+
   public:
+
     LineOpt();
     LineOpt(StateIn&);
     LineOpt(const Ref<KeyVal>&);
     ~LineOpt();
     void save_data_state(StateOut&);
 
-    void apply_tranform(const Ref<NonlinearTransform>&);
+    /** Initializes the line search object. Argument is a search direction.
+      * Use of this method assumes the Optimize base class already has a 
+      * function object (got it from a keyval or elsewhere). */
+    void init(RefSCVector& direction);
+    /** Initializes the line search object. First argument is a search 
+      * direction, second argument is a function object to optimize.
+      * Use this method when a function must be passed to the Optimize 
+      * base class. */
+    void init(RefSCVector& direction, Ref<Function> function);
+    /// Applies a nonlinear transform.
+    void apply_transform(const Ref<NonlinearTransform>&);
+  
+    /// Returns factor for sufficient decrease test
+    double decrease_factor() { return decrease_factor_; }
+    /// Sets factor for sufficient decrease test
+    double set_decrease_factor( double factor ) 
+    { double temp = decrease_factor_; decrease_factor_ = factor; return temp; }
+};
+
+class Backtrack: public LineOpt {
+
+ protected:
+   double backtrack_factor_;
+
+ public:
+   Backtrack(const Ref<KeyVal>&);
+   ~Backtrack(){}
+   int update();
 
-    void set_search_direction(RefSCVector&);
 };
 
 }
