@@ -50,6 +50,12 @@ static ClassDesc QNewtonOpt_cd(
 QNewtonOpt::QNewtonOpt(const Ref<KeyVal>&keyval):
   Optimize(keyval)
 {
+
+  if (function_.null()) {
+      ExEnv::err0() << "QNewtonOpt requires a function keyword" << endl;
+      abort();
+  }
+
   init();
 
   update_ << keyval->describedclassvalue("update");
@@ -60,6 +66,16 @@ QNewtonOpt::QNewtonOpt(const Ref<KeyVal>&keyval):
   print_x_ = keyval->booleanvalue("print_x");
   print_hessian_ = keyval->booleanvalue("print_hessian");
   print_gradient_ = keyval->booleanvalue("print_gradient");
+  linear_ = keyval->booleanvalue("linear");
+  if (keyval->error() != KeyVal::OK) linear_ = 0;
+  restrict_ = keyval->booleanvalue("restrict");
+  if (keyval->error() != KeyVal::OK) restrict_ = 1;
+  dynamic_grad_acc_ = keyval->booleanvalue("dynamic_grad_acc");
+  if (keyval->error() != KeyVal::OK) dynamic_grad_acc_ = 1;
+  force_search_ = keyval->booleanvalue("force_search");
+  if (keyval->error() != KeyVal::OK) force_search_ = 0;
+  restart_ = keyval->booleanvalue("restart");
+  if (keyval->error() != KeyVal::OK) restart_ = 1;
 
   RefSymmSCMatrix hessian(dimension(),matrixkit());
   // get a guess hessian from the function
@@ -137,71 +153,155 @@ QNewtonOpt::update()
   const double maxabs_gradient_to_desired_accuracy = 0.05;
   const double maxabs_gradient_to_next_desired_accuracy = 0.005;
   const double roundoff_error_factor = 1.1;
-
+  
   // the gradient convergence criterion.
   double old_maxabs_gradient = maxabs_gradient;
   RefSCVector xcurrent;
   RefSCVector gcurrent;
 
-  // get the next gradient at the required level of accuracy.
-  // usually only one pass is needed, unless we happen to find
-  // that the accuracy was set too low.
-  int accurate_enough;
-  do {
-      // compute the current point
-      function()->set_desired_gradient_accuracy(accuracy_);
+  if( dynamic_grad_acc_ ) {
+    // get the next gradient at the required level of accuracy.
+    // usually only one pass is needed, unless we happen to find
+    // that the accuracy was set too low.
+    int accurate_enough;
+    do {
+        // compute the current point
+        function()->set_desired_gradient_accuracy(accuracy_);
+        xcurrent = function()->get_x();
+        gcurrent = function()->gradient().copy();
+      
+        // compute the gradient convergence criterion now so i can see if
+        // the accuracy needs to be tighter
+        maxabs_gradient = gcurrent.maxabs();
+        // compute the required accuracy
+        accuracy_ = maxabs_gradient * maxabs_gradient_to_desired_accuracy;
+      
+        if (accuracy_ < DBL_EPSILON) accuracy_ = DBL_EPSILON;
 
-      xcurrent = function()->get_x();
-      gcurrent = function()->gradient().copy();
+        // The roundoff_error_factor is thrown in to allow for round off making
+        // the current gcurrent.maxabs() a bit smaller than the previous,
+        // which would make the current required accuracy less than the
+        // gradient's actual accuracy and cause everything to be recomputed.
+        accurate_enough = (
+            function()->actual_gradient_accuracy()
+            <= accuracy_*roundoff_error_factor);
 
-      // compute the gradient convergence criterion now so i can see if
-      // the accuracy needs to be tighter
-      maxabs_gradient = gcurrent.maxabs();
-      // compute the required accuracy
-      accuracy_ = maxabs_gradient * maxabs_gradient_to_desired_accuracy;
-
-      if (accuracy_ < DBL_EPSILON) accuracy_ = DBL_EPSILON;
-
-      // The roundoff_error_factor is thrown in to allow for round off making
-      // the current gcurrent.maxabs() a bit smaller than the previous,
-      // which would make the current required accuracy less than the
-      // gradient's actual accuracy and cause everything to be recomputed.
-      accurate_enough = (
-          function()->actual_gradient_accuracy()
-          <= accuracy_*roundoff_error_factor);
-
-      if (!accurate_enough) {
-        ExEnv::out0().unsetf(ios::fixed);
-        ExEnv::out0() << indent
-             << "NOTICE: function()->actual_gradient_accuracy() > accuracy_:\n"
-             << indent
-             << scprintf(
-               "        function()->actual_gradient_accuracy() = %15.8e",
-               function()->actual_gradient_accuracy()) << endl << indent
-             << scprintf(
-               "                                     accuracy_ = %15.8e",
-               accuracy_) << endl;
-      }
-    } while(!accurate_enough);
+        if (!accurate_enough) {
+          ExEnv::out0().unsetf(ios::fixed);
+          ExEnv::out0() << indent << 
+	       "NOTICE: function()->actual_gradient_accuracy() > accuracy_:\n"
+               << indent
+               << scprintf(
+                 "        function()->actual_gradient_accuracy() = %15.8e",
+                 function()->actual_gradient_accuracy()) << endl << indent
+               << scprintf(
+                 "                                     accuracy_ = %15.8e",
+                 accuracy_) << endl;
+        }
+     } while(!accurate_enough);
+    // increase accuracy, since the next gradient will be smaller
+    accuracy_ = maxabs_gradient * maxabs_gradient_to_next_desired_accuracy;
+  }
+  else {
+    xcurrent = function()->get_x();
+    gcurrent = function()->gradient().copy();
+  }
 
   if (old_maxabs_gradient >= 0.0 && old_maxabs_gradient < maxabs_gradient) {
     ExEnv::out0() << indent
-         << scprintf("NOTICE: maxabs_gradient increased from %8.4e to %8.4e",
+	 << scprintf("NOTICE: maxabs_gradient increased from %8.4e to %8.4e",
                      old_maxabs_gradient, maxabs_gradient) << endl;
   }
 
-  if (!take_newton_step_ && lineopt_.nonnull()) {
-      // see if the line min step is really needed
-//       if (line min really needed) {
-//           take_newton_step_ = (lineopt_->update() == 1);
-//           // maybe check for convergence and return here
-//         }
+  // update the hessian
+  if(update_.nonnull())
+    update_->update(ihessian_,function(),xcurrent,gcurrent);
+
+  conv_->reset();
+  conv_->get_grad(function());
+  conv_->get_x(function());
+
+  // compute the quadratic step
+  RefSCVector xdisp = -1.0*(ihessian_ * gcurrent);
+  RefSCVector xnext = xcurrent + xdisp;
+
+  // either do a lineopt or check stepsize
+  double tot;
+  if(lineopt_.nonnull()) {
+
+    // perform a search
+    double factor;
+    if( n_iterations_ == 0 && force_search_ ) 
+      factor = lineopt_->set_decrease_factor(1.0);
+    lineopt_->init(xdisp,function());
+    // reset value acc here so line search "precomputes" are 
+    // accurate enough for subsequent gradient evals
+    function()->set_desired_value_accuracy(accuracy_/100);
+    int acceptable = lineopt_->update();
+    if( n_iterations_ == 0 && force_search_ )
+      lineopt_->set_decrease_factor( factor );
+
+    if( !acceptable ) {
+      if( force_search_ ) factor = lineopt_->set_decrease_factor(1.0);
+
+      // try a new guess hessian
+      if( restart_ ) {
+	ExEnv::out0() << endl << indent << 
+	  "Restarting Hessian approximation" << endl;
+	RefSymmSCMatrix hessian(dimension(),matrixkit());
+	function()->guess_hessian(hessian);
+	ihessian_ = function()->inverse_hessian(hessian);
+	xdisp = -1.0 * (ihessian_ * gcurrent);
+	lineopt_->init(xdisp,function());
+	acceptable = lineopt_->update();
+      }
+      
+      // try steepest descent direction
+      if( !acceptable ) {
+	ExEnv::out0() << endl << indent << 
+	  "Trying steepest descent direction." << endl;
+	xdisp = -1.0 * gcurrent;
+	lineopt_->init(xdisp,function());
+	acceptable = lineopt_->update();
+      }
+
+      // give up and use steepest descent step
+      if( !acceptable ) {
+	ExEnv::out0() << endl << indent << 
+	  "Resorting to unscaled steepest descent step." << endl;
+	function()->set_x(xcurrent + xdisp);
+	Ref<NonlinearTransform> t = function()->change_coordinates();
+	apply_transform(t);
+      }
+
+      if( force_search_ ) lineopt_->set_decrease_factor( factor );
     }
 
-  // update the hessian
-  if (update_.nonnull()) {
-      update_->update(ihessian_,function(),xcurrent,gcurrent);
+    xnext = function()->get_x();
+    xdisp = xnext - xcurrent;
+    tot = sqrt(xdisp.scalar_product(xdisp));
+  }
+  else {
+
+    tot = sqrt(xdisp.scalar_product(xdisp));
+
+    if ( tot > max_stepsize_ ) {
+      if( restrict_ ) {
+	double scal = max_stepsize_/tot;
+	ExEnv::out0() << endl << indent <<
+	  scprintf("stepsize of %f is too big, scaling by %f",tot,scal)
+	  << endl;
+	xdisp.scale(scal);
+	tot *= scal;
+      }
+      else {
+	ExEnv::out0() << endl << indent <<
+	  scprintf("stepsize of %f is too big, but scaling is disabled",tot)
+	  << endl;
+      }
     }
+    xnext = xcurrent + xdisp;
+  }
 
   if (print_hessian_) {
     RefSymmSCMatrix hessian = ihessian_.gi();
@@ -235,45 +335,28 @@ QNewtonOpt::update()
     ExEnv::out0() << " ]" << endl;
   }
 
-  // take the step
-  RefSCVector xdisp = -1.0*(ihessian_ * gcurrent);
-  // scale the displacement vector if it's too large
-  double tot = sqrt(xdisp.scalar_product(xdisp));
-  if (tot > max_stepsize_) {
-    double scal = max_stepsize_/tot;
-    ExEnv::out0() << endl << indent
-         << scprintf("stepsize of %f is too big, scaling by %f",tot,scal)
-         << endl;
-    xdisp.scale(scal);
-    tot *= scal;
+  // check for convergence
+  conv_->set_nextx(xnext);
+  int converged = conv_->converged();
+  if (converged) return converged;
+
+  ExEnv::out0() << indent
+		<< scprintf("taking step of size %f", tot) << endl;
+  ExEnv::out0() << indent << "Optimization iteration " 
+		<< n_iterations_ + 1 << " complete" << endl;
+  ExEnv::out0() << indent
+    << "//////////////////////////////////////////////////////////////////////"
+    << endl;
+
+  if( lineopt_.null() ) {
+    function()->set_x(xnext);
+    Ref<NonlinearTransform> t = function()->change_coordinates();
+    apply_transform(t);
   }
 
-  RefSCVector xnext = xcurrent + xdisp;
+  if( dynamic_grad_acc_ ) 
+    function()->set_desired_gradient_accuracy(accuracy_);
 
-  conv_->reset();
-  conv_->get_grad(function());
-  conv_->get_x(function());
-  conv_->set_nextx(xnext);
-
-  // check for convergence before resetting the geometry
-  int converged = conv_->converged();
-  if (converged)
-    return converged;
-
-  ExEnv::out0() << endl << indent
-       << scprintf("taking step of size %f", tot) << endl;
-  
-  function()->set_x(xnext);
-  Ref<NonlinearTransform> t = function()->change_coordinates();
-  apply_transform(t);
-
-  // do a line min step next time
-  if (lineopt_.nonnull()) take_newton_step_ = 0;
-
-  // make the next gradient computed more accurate, since it will
-  // be smaller
-  accuracy_ = maxabs_gradient * maxabs_gradient_to_next_desired_accuracy;
-  
   return converged;
 }
 
